=== added file 'bzrlib/cmdline.py'
--- bzrlib/cmdline.py	1970-01-01 00:00:00 +0000
+++ bzrlib/cmdline.py	2010-02-02 04:37:56 +0000
@@ -0,0 +1,151 @@
+# Copyright (C) 2010 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
+
+"""Command-line parser for all platforms."""
+
+import re
+
+
+_whitespace_match = re.compile(u'\s', re.UNICODE).match
+
+
+class _PushbackSequence(object):
+    def __init__(self, orig):
+        self._iter = iter(orig)
+        self._pushback_buffer = []
+        
+    def next(self):
+        if len(self._pushback_buffer) > 0:
+            return self._pushback_buffer.pop()
+        else:
+            return self._iter.next()
+    
+    def pushback(self, char):
+        self._pushback_buffer.append(char)
+        
+    def __iter__(self):
+        return self
+
+
+class _Whitespace(object):
+    def process(self, next_char, seq, context):
+        if _whitespace_match(next_char):
+            if len(context.token) > 0:
+                return None
+            else:
+                return self
+        elif (next_char == u'"'
+              or (context.single_quotes_allowed and next_char == u"'")):
+            context.quoted = True
+            return _Quotes(next_char, self)
+        elif next_char == u'\\':
+            return _Backslash(self)
+        else:
+            context.token.append(next_char)
+            return _Word()
+
+
+class _Quotes(object):
+    def __init__(self, quote_char, exit_state):
+        self.quote_char = quote_char
+        self.exit_state = exit_state
+
+    def process(self, next_char, seq, context):
+        if next_char == u'\\':
+            return _Backslash(self)
+        elif next_char == self.quote_char:
+            return self.exit_state
+        else:
+            context.token.append(next_char)
+            return self
+
+
+class _Backslash(object):
+    # See http://msdn.microsoft.com/en-us/library/bb776391(VS.85).aspx
+    def __init__(self, exit_state):
+        self.exit_state = exit_state
+        self.count = 1
+        
+    def process(self, next_char, seq, context):
+        if next_char == u'\\':
+            self.count += 1
+            return self
+        elif next_char == u'"':
+            # 2N backslashes followed by '"' are N backslashes
+            context.token.append(u'\\' * (self.count/2))
+            # 2N+1 backslashes follwed by '"' are N backslashes followed by '"'
+            # which should not be processed as the start or end of quoted arg
+            if self.count % 2 == 1:
+                context.token.append(next_char) # odd number of '\' escapes the '"'
+            else:
+                seq.pushback(next_char) # let exit_state handle next_char
+            self.count = 0
+            return self.exit_state
+        else:
+            # N backslashes not followed by '"' are just N backslashes
+            if self.count > 0:
+                context.token.append(u'\\' * self.count)
+                self.count = 0
+            seq.pushback(next_char) # let exit_state handle next_char
+            return self.exit_state
+    
+    def finish(self, context):
+        if self.count > 0:
+            context.token.append(u'\\' * self.count)
+
+
+class _Word(object):
+    def process(self, next_char, seq, context):
+        if _whitespace_match(next_char):
+            return None
+        elif (next_char == u'"'
+              or (context.single_quotes_allowed and next_char == u"'")):
+            return _Quotes(next_char, self)
+        elif next_char == u'\\':
+            return _Backslash(self)
+        else:
+            context.token.append(next_char)
+            return self
+
+
+class Parser(object):
+    def __init__(self, command_line, single_quotes_allowed=False):
+        self._seq = _PushbackSequence(command_line)
+        self.single_quotes_allowed = single_quotes_allowed
+    
+    def __iter__(self):
+        return self
+    
+    def next(self):
+        quoted, token = self._get_token()
+        if token is None:
+            raise StopIteration
+        return quoted, token
+    
+    def _get_token(self):
+        self.quoted = False
+        self.token = []
+        state = _Whitespace()
+        for next_char in self._seq:
+            state = state.process(next_char, self._seq, self)
+            if state is None:
+                break
+        if not state is None and not getattr(state, 'finish', None) is None:
+            state.finish(self)
+        result = u''.join(self.token)
+        if not self.quoted and result == '':
+            result = None
+        return self.quoted, result

=== added file 'bzrlib/tests/test_cmdline.py'
--- bzrlib/tests/test_cmdline.py	1970-01-01 00:00:00 +0000
+++ bzrlib/tests/test_cmdline.py	2010-02-02 04:37:56 +0000
@@ -0,0 +1,74 @@
+from bzrlib import cmdline
+
+class TestParser(tests.TestCase):
+
+    def assertAsTokens(self, expected, line, single_quotes_allowed=False):
+        s = cmdline.Parser(line, single_quotes_allowed=single_quotes_allowed)
+        self.assertEqual(expected, list(s))
+
+    def test_simple(self):
+        self.assertAsTokens([(False, u'foo'), (False, u'bar'), (False, u'baz')],
+                            u'foo bar baz')
+
+    def test_ignore_multiple_spaces(self):
+        self.assertAsTokens([(False, u'foo'), (False, u'bar')], u'foo  bar')
+
+    def test_ignore_leading_space(self):
+        self.assertAsTokens([(False, u'foo'), (False, u'bar')], u'  foo bar')
+
+    def test_ignore_trailing_space(self):
+        self.assertAsTokens([(False, u'foo'), (False, u'bar')], u'foo bar  ')
+
+    def test_posix_quotations(self):
+        self.assertAsTokens([(True, u'foo bar')], u"'foo bar'",
+            single_quotes_allowed=True)
+        self.assertAsTokens([(True, u'foo bar')], u"'fo''o b''ar'",
+            single_quotes_allowed=True)
+        self.assertAsTokens([(True, u'foo bar')], u'"fo""o b""ar"',
+            single_quotes_allowed=True)
+        self.assertAsTokens([(True, u'foo bar')], u'"fo"\'o b\'"ar"',
+            single_quotes_allowed=True)
+
+    def test_nested_quotations(self):
+        self.assertAsTokens([(True, u'foo"" bar')], u"\"foo\\\"\\\" bar\"")
+        self.assertAsTokens([(True, u'foo\'\' bar')], u"\"foo'' bar\"")
+        self.assertAsTokens([(True, u'foo\'\' bar')], u"\"foo'' bar\"",
+            single_quotes_allowed=True)
+        self.assertAsTokens([(True, u'foo"" bar')], u"'foo\"\" bar'",
+            single_quotes_allowed=True)
+
+    def test_empty_result(self):
+        self.assertAsTokens([], u'')
+        self.assertAsTokens([], u'    ')
+
+    def test_quoted_empty(self):
+        self.assertAsTokens([(True, '')], u'""')
+        self.assertAsTokens([(False, u"''")], u"''")
+        self.assertAsTokens([(True, '')], u"''", single_quotes_allowed=True)
+
+    def test_unicode_chars(self):
+        self.assertAsTokens([(False, u'f\xb5\xee'), (False, u'\u1234\u3456')],
+                             u'f\xb5\xee \u1234\u3456')
+
+    def test_newline_in_quoted_section(self):
+        self.assertAsTokens([(True, u'foo\nbar\nbaz\n')], u'"foo\nbar\nbaz\n"')
+        self.assertAsTokens([(True, u'foo\nbar\nbaz\n')], u"'foo\nbar\nbaz\n'",
+            single_quotes_allowed=True)
+
+    def test_escape_chars(self):
+        self.assertAsTokens([(False, u'foo\\bar')], u'foo\\bar')
+
+    def test_escape_quote(self):
+        self.assertAsTokens([(True, u'foo"bar')], u'"foo\\"bar"')
+        self.assertAsTokens([(True, u'foo\\"bar')], u'"foo\\\\\\"bar"')
+        self.assertAsTokens([(True, u'foo\\bar')], u'"foo\\\\"bar"')
+
+    def test_double_escape(self):
+        self.assertAsTokens([(True, u'foo\\\\bar')], u'"foo\\\\bar"')
+        self.assertAsTokens([(False, u'foo\\\\bar')], u"foo\\\\bar")
+        
+    def test_multiple_quoted_args(self):
+        self.assertAsTokens([(True, u'x x'), (True, u'y y')],
+            u'"x x" "y y"')
+        self.assertAsTokens([(True, u'x x'), (True, u'y y')],
+            u'"x x" \'y y\'', single_quotes_allowed=True)

=== modified file 'bzrlib/tests/test_win32utils.py'
--- bzrlib/tests/test_win32utils.py	2010-02-01 00:50:54 +0000
+++ bzrlib/tests/test_win32utils.py	2010-02-02 04:37:56 +0000
@@ -287,79 +287,6 @@
         win32utils.set_file_attr_hidden(path)
 
 
-class TestUnicodeShlex(tests.TestCase):
-
-    def assertAsTokens(self, expected, line, single_quotes_allowed=False):
-        s = win32utils.UnicodeShlex(line,
-                                    single_quotes_allowed=single_quotes_allowed)
-        self.assertEqual(expected, list(s))
-
-    def test_simple(self):
-        self.assertAsTokens([(False, u'foo'), (False, u'bar'), (False, u'baz')],
-                            u'foo bar baz')
-
-    def test_ignore_multiple_spaces(self):
-        self.assertAsTokens([(False, u'foo'), (False, u'bar')], u'foo  bar')
-
-    def test_ignore_leading_space(self):
-        self.assertAsTokens([(False, u'foo'), (False, u'bar')], u'  foo bar')
-
-    def test_ignore_trailing_space(self):
-        self.assertAsTokens([(False, u'foo'), (False, u'bar')], u'foo bar  ')
-
-    def test_posix_quotations(self):
-        self.assertAsTokens([(True, u'foo bar')], u"'foo bar'",
-            single_quotes_allowed=True)
-        self.assertAsTokens([(True, u'foo bar')], u"'fo''o b''ar'",
-            single_quotes_allowed=True)
-        self.assertAsTokens([(True, u'foo bar')], u'"fo""o b""ar"',
-            single_quotes_allowed=True)
-        self.assertAsTokens([(True, u'foo bar')], u'"fo"\'o b\'"ar"',
-            single_quotes_allowed=True)
-
-    def test_nested_quotations(self):
-        self.assertAsTokens([(True, u'foo"" bar')], u"\"foo\\\"\\\" bar\"")
-        self.assertAsTokens([(True, u'foo\'\' bar')], u"\"foo'' bar\"")
-        self.assertAsTokens([(True, u'foo\'\' bar')], u"\"foo'' bar\"",
-            single_quotes_allowed=True)
-        self.assertAsTokens([(True, u'foo"" bar')], u"'foo\"\" bar'",
-            single_quotes_allowed=True)
-
-    def test_empty_result(self):
-        self.assertAsTokens([], u'')
-        self.assertAsTokens([], u'    ')
-
-    def test_quoted_empty(self):
-        self.assertAsTokens([(True, '')], u'""')
-        self.assertAsTokens([(False, u"''")], u"''")
-        self.assertAsTokens([(True, '')], u"''", single_quotes_allowed=True)
-
-    def test_unicode_chars(self):
-        self.assertAsTokens([(False, u'f\xb5\xee'), (False, u'\u1234\u3456')],
-                             u'f\xb5\xee \u1234\u3456')
-
-    def test_newline_in_quoted_section(self):
-        self.assertAsTokens([(True, u'foo\nbar\nbaz\n')], u'"foo\nbar\nbaz\n"')
-        self.assertAsTokens([(True, u'foo\nbar\nbaz\n')], u"'foo\nbar\nbaz\n'",
-            single_quotes_allowed=True)
-
-    def test_escape_chars(self):
-        self.assertAsTokens([(False, u'foo\\bar')], u'foo\\bar')
-
-    def test_escape_quote(self):
-        self.assertAsTokens([(True, u'foo"bar')], u'"foo\\"bar"')
-        self.assertAsTokens([(True, u'foo\\"bar')], u'"foo\\\\\\"bar"')
-        self.assertAsTokens([(True, u'foo\\bar')], u'"foo\\\\"bar"')
-
-    def test_double_escape(self):
-        self.assertAsTokens([(True, u'foo\\\\bar')], u'"foo\\\\bar"')
-        self.assertAsTokens([(False, u'foo\\\\bar')], u"foo\\\\bar")
-        
-    def test_multiple_quoted_args(self):
-        self.assertAsTokens([(True, u'x x'), (True, u'y y')],
-            u'"x x" "y y"')
-        self.assertAsTokens([(True, u'x x'), (True, u'y y')],
-            u'"x x" \'y y\'', single_quotes_allowed=True)
 
 
 class Test_CommandLineToArgv(tests.TestCaseInTempDir):

=== modified file 'bzrlib/win32utils.py'
--- bzrlib/win32utils.py	2010-01-15 13:21:13 +0000
+++ bzrlib/win32utils.py	2010-02-02 04:37:56 +0000
@@ -25,6 +25,7 @@
 import struct
 import sys
 
+from bzrlib import cmdline
 
 # Windows version
 if sys.platform == 'win32':
@@ -517,138 +518,6 @@
             trace.mutter('Unable to set hidden attribute on %r: %s', path, e)
 
 
-_whitespace_match = re.compile(u'\s', re.UNICODE).match
-
-
-class _PushbackSequence(object):
-    def __init__(self, orig):
-        self._iter = iter(orig)
-        self._pushback_buffer = []
-        
-    def next(self):
-        if len(self._pushback_buffer) > 0:
-            return self._pushback_buffer.pop()
-        else:
-            return self._iter.next()
-    
-    def pushback(self, char):
-        self._pushback_buffer.append(char)
-        
-    def __iter__(self):
-        return self
-
-
-class _Whitespace(object):
-    def process(self, next_char, seq, context):
-        if _whitespace_match(next_char):
-            if len(context.token) > 0:
-                return None
-            else:
-                return self
-        elif (next_char == u'"'
-              or (context.single_quotes_allowed and next_char == u"'")):
-            context.quoted = True
-            return _Quotes(next_char, self)
-        elif next_char == u'\\':
-            return _Backslash(self)
-        else:
-            context.token.append(next_char)
-            return _Word()
-
-
-class _Quotes(object):
-    def __init__(self, quote_char, exit_state):
-        self.quote_char = quote_char
-        self.exit_state = exit_state
-
-    def process(self, next_char, seq, context):
-        if next_char == u'\\':
-            return _Backslash(self)
-        elif next_char == self.quote_char:
-            return self.exit_state
-        else:
-            context.token.append(next_char)
-            return self
-
-
-class _Backslash(object):
-    # See http://msdn.microsoft.com/en-us/library/bb776391(VS.85).aspx
-    def __init__(self, exit_state):
-        self.exit_state = exit_state
-        self.count = 1
-        
-    def process(self, next_char, seq, context):
-        if next_char == u'\\':
-            self.count += 1
-            return self
-        elif next_char == u'"':
-            # 2N backslashes followed by '"' are N backslashes
-            context.token.append(u'\\' * (self.count/2))
-            # 2N+1 backslashes follwed by '"' are N backslashes followed by '"'
-            # which should not be processed as the start or end of quoted arg
-            if self.count % 2 == 1:
-                context.token.append(next_char) # odd number of '\' escapes the '"'
-            else:
-                seq.pushback(next_char) # let exit_state handle next_char
-            self.count = 0
-            return self.exit_state
-        else:
-            # N backslashes not followed by '"' are just N backslashes
-            if self.count > 0:
-                context.token.append(u'\\' * self.count)
-                self.count = 0
-            seq.pushback(next_char) # let exit_state handle next_char
-            return self.exit_state
-    
-    def finish(self, context):
-        if self.count > 0:
-            context.token.append(u'\\' * self.count)
-
-
-class _Word(object):
-    def process(self, next_char, seq, context):
-        if _whitespace_match(next_char):
-            return None
-        elif (next_char == u'"'
-              or (context.single_quotes_allowed and next_char == u"'")):
-            return _Quotes(next_char, self)
-        elif next_char == u'\\':
-            return _Backslash(self)
-        else:
-            context.token.append(next_char)
-            return self
-
-
-class UnicodeShlex(object):
-    def __init__(self, command_line, single_quotes_allowed=False):
-        self._seq = _PushbackSequence(command_line)
-        self.single_quotes_allowed = single_quotes_allowed
-    
-    def __iter__(self):
-        return self
-    
-    def next(self):
-        quoted, token = self._get_token()
-        if token is None:
-            raise StopIteration
-        return quoted, token
-    
-    def _get_token(self):
-        self.quoted = False
-        self.token = []
-        state = _Whitespace()
-        for next_char in self._seq:
-            state = state.process(next_char, self._seq, self)
-            if state is None:
-                break
-        if not state is None and not getattr(state, 'finish', None) is None:
-            state.finish(self)
-        result = u''.join(self.token)
-        if not self.quoted and result == '':
-            result = None
-        return self.quoted, result
-
-
 def command_line_to_argv(command_line, wildcard_expansion=True,
                          single_quotes_allowed=False):
     """Convert a Unicode command line into a list of argv arguments.
@@ -665,7 +534,7 @@
                                   default.
     :return: A list of unicode strings.
     """
-    s = UnicodeShlex(command_line, single_quotes_allowed=single_quotes_allowed)
+    s = cmdline.Parser(command_line, single_quotes_allowed=single_quotes_allowed)
     # Now that we've split the content, expand globs if necessary
     # TODO: Use 'globbing' instead of 'glob.glob', this gives us stuff like
     #       '**/' style globs

